@@ -32,15 +32,13 @@
 #include <pub_tool_threadstate.h>
 #include <pub_tool_libcfile.h>
 
-/*------------------------------------------------------------*/
-/*--- Support for signal handlers and multi-threading      ---*/
-/*------------------------------------------------------------*/
 
 /* Dump Part Counter */
 static Int out_counter = 0;
 
-static Char* dump_file_base = 0;
-static Char* base_directory = 0;
+static Char* out_file = 0;
+static Char* out_directory = 0;
+static Bool dumps_initialized = False;
 
 /* Command */
 static Char cmdbuf[BUF_LEN];
@@ -64,9 +62,16 @@ Int CLG_(get_dump_counter)(void)
   return out_counter;
 }
 
-Char* CLG_(get_dump_file_base)()
+Char* CLG_(get_out_file)()
 {
-  return dump_file_base;
+    CLG_ASSERT(dumps_initialized);
+    return out_file;
+}
+
+Char* CLG_(get_out_directory)()
+{
+    CLG_ASSERT(dumps_initialized);
+    return out_directory;
 }
 
 /*------------------------------------------------------------*/
@@ -429,31 +434,40 @@ void init_debug_cache(void)
     }
 }
 
-static __inline__
+static /* __inline__ */
 Bool get_debug_pos(BBCC* bbcc, Addr addr, AddrPos* p)
 {
     Char file[FILENAME_LEN];
-    Bool res;
+    Char dir[FILENAME_LEN];
+    Bool found_file_line, found_dirname;
 
     int cachepos = addr % DEBUG_CACHE_SIZE;
     
     if (debug_cache_addr[cachepos] == addr) {
 	p->line = debug_cache_line[cachepos];
 	p->file = debug_cache_file[cachepos];
-	res     = debug_cache_info[cachepos];
+	found_file_line = debug_cache_info[cachepos];
     }
     else {
-	res = VG_(get_filename_linenum)(addr,
-					file, FILENAME_LEN,
-					NULL, 0, NULL, //FIXME
-					&(p->line));
-	if (!res) {
+	found_file_line = VG_(get_filename_linenum)(addr,
+						    file, FILENAME_LEN,
+						    dir, FILENAME_LEN,
+						    &found_dirname,
+						    &(p->line));
+	if (!found_file_line) {
 	    VG_(strcpy)(file, "???");
 	    p->line = 0;
 	}
+	if (found_dirname) {
+	    // +1 for the '/'.
+	    CLG_ASSERT(VG_(strlen)(dir) + VG_(strlen)(file) + 1 < FILENAME_LEN);
+	    VG_(strcat)(dir, "/");     // Append '/'
+	    VG_(strcat)(dir, file);    // Append file to dir
+	    VG_(strcpy)(file, dir);    // Move dir+file to file
+	}
 	p->file    = CLG_(get_file_node)(bbcc->bb->obj, file);
 
-	debug_cache_info[cachepos] = res;
+	debug_cache_info[cachepos] = found_file_line;
 	debug_cache_addr[cachepos] = addr;
 	debug_cache_line[cachepos] = p->line;
 	debug_cache_file[cachepos] = p->file;
@@ -467,7 +481,7 @@ Bool get_debug_pos(BBCC* bbcc, Addr addr, AddrPos* p)
 	     addr, bb_addr(bbcc->bb), bbcc->cxt->fn[0]->name,
 	     p->file->name, p->line);
 
-    return res;
+    return found_file_line;
 }
 
 
@@ -540,7 +554,7 @@ static
 void fprint_pos(Int fd, AddrPos* curr, AddrPos* last)
 {
     if (0) //CLG_(clo).dump_bbs)
-	VG_(sprintf)(outbuf, "%u ", curr->addr - curr->bb_addr);
+	VG_(sprintf)(outbuf, "%lu ", curr->addr - curr->bb_addr);
     else {
 	int p = 0;
 	if (CLG_(clo).dump_instr) {
@@ -1264,10 +1278,11 @@ static int new_dumpfile(Char buf[BUF_LEN], int tid, Char* trigger)
     FullCost sum = 0;
     SysRes res;
 
+    CLG_ASSERT(dumps_initialized);
     CLG_ASSERT(filename != 0);
 
     if (!CLG_(clo).combine_dumps) {
-	i = VG_(sprintf)(filename, "%s.%d", dump_file_base, VG_(getpid)());
+	i = VG_(sprintf)(filename, "%s", out_file);
     
 	if (trigger)
 	    i += VG_(sprintf)(filename+i, ".%d", out_counter);
@@ -1278,7 +1293,7 @@ static int new_dumpfile(Char buf[BUF_LEN], int tid, Char* trigger)
 	res = VG_(open)(filename, VKI_O_WRONLY|VKI_O_TRUNC, 0);
     }
     else {
-	VG_(sprintf)(filename, "%s.%d", dump_file_base, VG_(getpid)());
+	VG_(sprintf)(filename, "%s", out_file);
         res = VG_(open)(filename, VKI_O_WRONLY|VKI_O_APPEND, 0);
 	if (!res.isError && out_counter>1)
 	    appending = True;
@@ -1293,7 +1308,7 @@ static int new_dumpfile(Char buf[BUF_LEN], int tid, Char* trigger)
 	    file_err();
 	}
     }
-    fd = (Int) res.val;
+    fd = (Int) res.res;
 
     CLG_DEBUG(2, "  new_dumpfile '%s'\n", filename);
 
@@ -1551,7 +1566,7 @@ static void print_bbccs_of_thread(thread_info* ti)
     
     p++;
   }
-  
+
   close_dumpfile(print_buf, print_fd, CLG_(current_tid));
   if (array) VG_(free)(array);
   
@@ -1602,7 +1617,6 @@ void CLG_(dump_profile)(Char* trigger, Bool only_current_thread)
 
    print_bbccs(trigger, only_current_thread);
 
-
    bbs_done = CLG_(stat).bb_executions++;
 
    if (VG_(clo_verbosity) > 1)
@@ -1620,8 +1634,8 @@ void init_cmdbuf(void)
   if (VG_(args_the_exename))
       size = VG_(sprintf)(cmdbuf, " %s", VG_(args_the_exename));
 
-  for(i = 0; i < VG_(args_for_client).used; i++) {
-      argv = VG_(args_for_client).strs[i];
+  for(i = 0; i < VG_(sizeXA)( VG_(args_for_client) ); i++) {
+      argv = * (HChar**) VG_(indexXA)( VG_(args_for_client), i );
       if (!argv) continue;
       if ((size>0) && (size < BUF_LEN)) cmdbuf[size++] = ' ';
       for(j=0;argv[j]!=0;j++)
@@ -1629,7 +1643,7 @@ void init_cmdbuf(void)
   }
 #else
   for(i = 0; i < VG_(client_argc); i++) {
-    argv = VG_(client_argv[i]);
+    argv = VG_(client_argv)[i];
     if (!argv) continue;
     if ((size>0) && (size < BUF_LEN)) cmdbuf[size++] = ' ';
     for(j=0;argv[j]!=0;j++)
@@ -1641,55 +1655,44 @@ void init_cmdbuf(void)
   cmdbuf[size] = 0;
 }
 
-void CLG_(init_files)(Char** dir, Char** file)
+/*
+ * Set up file names for dump output: <out_directory>, <out_file>.
+ * <out_file> is derived from the output format string, which defaults
+ * to "callgrind.out.%p", where %p is replaced with the PID.
+ * For the final file name, on intermediate dumps a counter is appended,
+ * and further, if separate dumps per thread are requested, the thread ID.
+ *
+ * <out_file> always starts with a full absolute path.
+ * If the output format string represents a relative path, the current
+ * working directory at program start is used.
+ */
+void CLG_(init_dumps)()
 {
-  Int size;
-  SysRes res;
+   Int lastSlash, i;
+   SysRes res;
 
-   if (!CLG_(clo).filename_base)
-     CLG_(clo).filename_base = DEFAULT_DUMPNAME;
+   if (!CLG_(clo).out_format)
+     CLG_(clo).out_format = DEFAULT_OUTFORMAT;
+
+   // Setup output filename.
+   out_file =
+       VG_(expand_file_name)("--callgrind-out-file", CLG_(clo).out_format);
 
    /* get base directory for dump/command/result files */
-   if (CLG_(clo).filename_base[0] == '/') {
-       int lastSlash = 0, i =1;
-       while(CLG_(clo).filename_base[i]) {
-	 for(; CLG_(clo).filename_base[i] &&
-	       CLG_(clo).filename_base[i] != '/'; i++);
-	   if (CLG_(clo).filename_base[i] != '/') break;
-	   lastSlash = i;
-	   i++;
-       }
-       i = lastSlash;
-       base_directory = (Char*) CLG_MALLOC(i+1);
-       VG_(strncpy)(base_directory, CLG_(clo).filename_base, i);
-       base_directory[i] = 0;
-
-       dump_file_base = CLG_(clo).filename_base;
-   }
-   else {
-       size = 100;
-       base_directory = 0;
-
-       /* getcwd() fails if the buffer isn't big enough -- keep doubling size
-          until it succeeds. */
-       while (NULL == base_directory) {
-           base_directory = CLG_MALLOC(size);
-           if (!VG_(getcwd)(base_directory, size)) {
-               VG_(free)(base_directory);
-               base_directory = 0;
-               size *= 2;
-           }
-       }
-
-       size = VG_(strlen)(base_directory) + VG_(strlen)(CLG_(clo).filename_base) +2;
-       dump_file_base = (Char*) CLG_MALLOC(size);
-       CLG_ASSERT(dump_file_base != 0);
-       VG_(sprintf)(dump_file_base, "%s/%s",
-		    base_directory, CLG_(clo).filename_base);
+   CLG_ASSERT(out_file[0] == '/');
+   lastSlash = 0;
+   i = 1;
+   while(out_file[i]) {
+       if (out_file[i] == '/') lastSlash = i;
+       i++;
    }
+   i = lastSlash;
+   out_directory = (Char*) CLG_MALLOC(i+1);
+   VG_(strncpy)(out_directory, out_file, i);
+   out_directory[i] = 0;
 
    /* allocate space big enough for final filenames */
-   filename = (Char*) CLG_MALLOC(VG_(strlen)(dump_file_base)+32);
+   filename = (Char*) CLG_MALLOC(VG_(strlen)(out_file)+32);
    CLG_ASSERT(filename != 0);
        
    /* Make sure the output base file can be written.
@@ -1698,7 +1701,7 @@ void CLG_(init_files)(Char** dir, Char** file)
     * file: This is probably because of missing rights,
     * and trace parts wouldn't be allowed to be written, too.
     */ 
-    VG_(sprintf)(filename, "%s.%d", dump_file_base, VG_(getpid)());
+    VG_(strcpy)(filename, out_file);
     res = VG_(open)(filename, VKI_O_WRONLY|VKI_O_TRUNC, 0);
     if (res.isError) { 
 	res = VG_(open)(filename, VKI_O_CREAT|VKI_O_WRONLY,
@@ -1707,10 +1710,9 @@ void CLG_(init_files)(Char** dir, Char** file)
 	    file_err(); 
 	}
     }
-    if (!res.isError) VG_(close)( (Int)res.val );
-
-    *dir  = base_directory;
-    *file = filename;
+    if (!res.isError) VG_(close)( (Int)res.res );
 
     init_cmdbuf();
+
+    dumps_initialized = True;
 }
