@@ -8,7 +8,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2006 Julian Seward 
+   Copyright (C) 2000-2007 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -35,6 +35,7 @@
 */
 
 #include "pub_core_basics.h"
+#include "pub_core_vki.h"
 #include "pub_core_libcbase.h"
 #include "pub_core_libcprint.h"
 #include "pub_core_libcassert.h"
@@ -45,6 +46,7 @@
 #include "pub_core_options.h"
 #include "pub_core_oset.h"
 #include "pub_core_tooliface.h"    /* VG_(needs) */
+#include "pub_core_xarray.h"
 #include "priv_storage.h"
 #include "priv_readelf.h"          /* self */
 #include "priv_readdwarf.h"        /* 'cos ELF contains DWARF */
@@ -232,9 +234,9 @@ Bool get_elf_symbol_info (
    if (!plausible
        && ELFXX_ST_TYPE(sym->st_info) == STT_NOTYPE
        && sym->st_size > 0
-       && si->opd_start_vma != 0
-       && sym_addr >= si->opd_start_vma
-       && sym_addr <  si->opd_start_vma + si->opd_size)
+       && si->opd_start_avma != 0
+       && sym_addr >= si->opd_start_avma
+       && sym_addr <  si->opd_start_avma + si->opd_size)
       plausible = True;
 #  endif
 
@@ -242,7 +244,7 @@ Bool get_elf_symbol_info (
       return False;
 
    /* Ignore if nameless, or zero-sized. */
-   if (sym->st_name == (ElfXX_Word)NULL
+   if (sym->st_name == (ElfXX_Word)0
        || /* VG_(strlen)(sym_name) == 0 */
           /* equivalent but cheaper ... */
           sym_name[0] == 0
@@ -261,15 +263,15 @@ Bool get_elf_symbol_info (
 
    /* If it's apparently in a GOT or PLT, it's really a reference to a
       symbol defined elsewhere, so ignore it. */
-   if (si->got_start_vma != 0
-       && sym_addr >= si->got_start_vma 
-       && sym_addr <  si->got_start_vma + si->got_size) {
+   if (si->got_start_avma != 0
+       && sym_addr >= si->got_start_avma 
+       && sym_addr <  si->got_start_avma + si->got_size) {
       TRACE_SYMTAB("    ignore -- in GOT: %s\n", sym_name);
       return False;
    }
-   if (si->plt_start_vma != 0
-       && sym_addr >= si->plt_start_vma
-       && sym_addr <  si->plt_start_vma + si->plt_size) {
+   if (si->plt_start_avma != 0
+       && sym_addr >= si->plt_start_avma
+       && sym_addr <  si->plt_start_avma + si->plt_size) {
       TRACE_SYMTAB("    ignore -- in PLT: %s\n", sym_name);
       return False;
    }
@@ -284,9 +286,9 @@ Bool get_elf_symbol_info (
    */
    is_in_opd = False;
 
-   if (si->opd_start_vma != 0
-       && sym_addr >= si->opd_start_vma
-       && sym_addr <  si->opd_start_vma + si->opd_size) {
+   if (si->opd_start_avma != 0
+       && sym_addr >= si->opd_start_avma
+       && sym_addr <  si->opd_start_avma + si->opd_size) {
 #     if !defined(VGP_ppc64_linux)
       TRACE_SYMTAB("    ignore -- in OPD: %s\n", sym_name);
       return False;
@@ -306,7 +308,7 @@ Bool get_elf_symbol_info (
          the vma of the opd section start, so we can figure out how
          far into the opd section this is. */
 
-      offset_in_opd = (Addr)sym_addr - (Addr)(si->opd_start_vma);
+      offset_in_opd = (Addr)sym_addr - (Addr)(si->opd_start_avma);
       if (offset_in_opd < 0 || offset_in_opd >= si->opd_size) {
          TRACE_SYMTAB("    ignore -- invalid OPD offset: %s\n", sym_name);
          return False;
@@ -345,7 +347,7 @@ Bool get_elf_symbol_info (
    /* Here's yet another ppc64-linux hack.  Get rid of leading dot if
       the symbol is outside .opd. */
 #  if defined(VGP_ppc64_linux)
-   if (si->opd_start_vma != 0
+   if (si->opd_start_avma != 0
        && !is_in_opd
        && sym_name[0] == '.') {
       vg_assert(!(*from_opd_out));
@@ -355,11 +357,12 @@ Bool get_elf_symbol_info (
 
    /* If no part of the symbol falls within the mapped range,
       ignore it. */
-   if (*sym_addr_out + *sym_size_out <= si->start
-       || *sym_addr_out >= si->start+si->size) {
+   if (*sym_addr_out + *sym_size_out <= si->text_start_avma
+       || *sym_addr_out >= si->text_start_avma + si->text_size) {
       TRACE_SYMTAB( "ignore -- %p .. %p outside mapped range %p .. %p\n",
                     *sym_addr_out, *sym_addr_out + *sym_size_out,
-                    si->start, si->start+si->size);
+                    si->text_start_avma,
+                    si->text_start_avma + si->text_size);
       return False;
    }
 
@@ -368,9 +371,9 @@ Bool get_elf_symbol_info (
       section.  This would completely mess up function redirection and
       intercepting.  This assert ensures that any symbols that make it
       into the symbol table on ppc64-linux don't point into .opd. */
-   if (si->opd_start_vma != 0) {
-      vg_assert(*sym_addr_out + *sym_size_out <= si->opd_start_vma
-                || *sym_addr_out >= si->opd_start_vma + si->opd_size);
+   if (si->opd_start_avma != 0) {
+      vg_assert(*sym_addr_out + *sym_size_out <= si->opd_start_avma
+                || *sym_addr_out >= si->opd_start_avma + si->opd_size);
    }
 #  endif
 
@@ -417,7 +420,7 @@ void read_elf_symtab__normal(
       sym_name = (Char*)(o_strtab + sym->st_name);
       sym_addr = o_symtab_offset + sym->st_value;
 
-      if (VG_(clo_trace_symtab))
+      if (si->trace_symtab)
          show_raw_elf_symbol(i, sym, sym_name, sym_addr, False);
 
       if (get_elf_symbol_info(si, sym, sym_name, sym_addr,
@@ -436,7 +439,7 @@ void read_elf_symtab__normal(
          vg_assert(risym.tocptr == 0); /* has no role except on ppc64-linux */
          ML_(addSym) ( si, &risym );
 
-         if (VG_(clo_trace_symtab)) {
+         if (si->trace_symtab) {
             VG_(printf)("    record [%4d]:          "
                         " val %010p, sz %4d  %s\n",
                         i, (void*)risym.addr, (Int)risym.size, 
@@ -513,9 +516,9 @@ void read_elf_symtab__ppc64_linux(
    TRACE_SYMTAB("\nReading (ELF, ppc64-linux) %s (%d entries)\n", tab_name, 
                 o_symtab_sz/sizeof(ElfXX_Sym) );
 
-   oset = VG_(OSet_Create)( offsetof(TempSym,key), 
-                            (OSetCmp_t)cmp_TempSymKey, 
-                            oset_malloc, oset_free );
+   oset = VG_(OSetGen_Create)( offsetof(TempSym,key), 
+                               (OSetCmp_t)cmp_TempSymKey, 
+                               oset_malloc, oset_free );
    vg_assert(oset);
 
    /* Perhaps should start at i = 1; ELF docs suggest that entry
@@ -525,7 +528,7 @@ void read_elf_symtab__ppc64_linux(
       sym_name = (Char*)(o_strtab + sym->st_name);
       sym_addr = o_symtab_offset + sym->st_value;
 
-      if (VG_(clo_trace_symtab))
+      if (si->trace_symtab)
          show_raw_elf_symbol(i, sym, sym_name, sym_addr, True);
 
       if (get_elf_symbol_info(si, sym, sym_name, sym_addr,
@@ -539,7 +542,7 @@ void read_elf_symtab__ppc64_linux(
          /* Check if we've seen this (name,addr) key before. */
          key.addr = sym_addr_really;
          key.name = sym_name_really;
-         prev = VG_(OSet_Lookup)( oset, &key );
+         prev = VG_(OSetGen_Lookup)( oset, &key );
 
          if (prev) {
 
@@ -578,7 +581,7 @@ void read_elf_symtab__ppc64_linux(
             /* Only one or the other is possible (I think) */
 	    vg_assert(!(modify_size && modify_tocptr));
 
-            if (modify_size && VG_(clo_trace_symtab)) {
+            if (modify_size && si->trace_symtab) {
                VG_(printf)("    modify (old sz %4d)    "
                            " val %010p, toc %010p, sz %4d  %s\n",
                            old_size,
@@ -588,7 +591,7 @@ void read_elf_symtab__ppc64_linux(
                            (HChar*)prev->key.name
                );
             }
-            if (modify_tocptr && VG_(clo_trace_symtab)) {
+            if (modify_tocptr && si->trace_symtab) {
                VG_(printf)("    modify (upd tocptr)     "
                            " val %010p, toc %010p, sz %4d  %s\n",
                             (void*) prev->key.addr, 
@@ -601,14 +604,14 @@ void read_elf_symtab__ppc64_linux(
          } else {
 
             /* A new (name,addr) key.  Add and continue. */
-            elem = VG_(OSet_AllocNode)(oset, sizeof(TempSym));
+            elem = VG_(OSetGen_AllocNode)(oset, sizeof(TempSym));
             vg_assert(elem);
             elem->key      = key;
             elem->tocptr   = sym_tocptr;
             elem->size     = sym_size;
             elem->from_opd = from_opd;
-            VG_(OSet_Insert)(oset, elem);
-            if (VG_(clo_trace_symtab)) {
+            VG_(OSetGen_Insert)(oset, elem);
+            if (si->trace_symtab) {
                VG_(printf)("   to-oset [%4d]:          "
                            " val %010p, toc %010p, sz %4d  %s\n",
                            i, (void*) elem->key.addr,
@@ -626,9 +629,9 @@ void read_elf_symtab__ppc64_linux(
       build a "standard" symbol table, and nuke the oset. */
 
    i = 0;
-   VG_(OSet_ResetIter)( oset );
+   VG_(OSetGen_ResetIter)( oset );
 
-   while ( (elem = VG_(OSet_Next)(oset)) ) {
+   while ( (elem = VG_(OSetGen_Next)(oset)) ) {
       risym.addr   = elem->key.addr;
       risym.size   = elem->size;
       risym.name   = ML_(addStr) ( si, elem->key.name, -1 );
@@ -636,7 +639,7 @@ void read_elf_symtab__ppc64_linux(
       vg_assert(risym.name != NULL);
 
       ML_(addSym) ( si, &risym );
-      if (VG_(clo_trace_symtab)) {
+      if (si->trace_symtab) {
          VG_(printf)("    record [%4d]:          "
                      " val %010p, toc %010p, sz %4d  %s\n",
                      i, (void*) risym.addr,
@@ -648,7 +651,7 @@ void read_elf_symtab__ppc64_linux(
       i++;
    }
 
-   VG_(OSet_Destroy)( oset, NULL );
+   VG_(OSetGen_Destroy)( oset );
 }
 
 
@@ -737,8 +740,8 @@ Addr open_debug_file( Char* name, UInt crc, UInt* size )
    if (fd.isError)
       return 0;
 
-   if (VG_(fstat)(fd.val, &stat_buf) != 0) {
-      VG_(close)(fd.val);
+   if (VG_(fstat)(fd.res, &stat_buf) != 0) {
+      VG_(close)(fd.res);
       return 0;
    }
 
@@ -748,23 +751,23 @@ Addr open_debug_file( Char* name, UInt crc, UInt* size )
    *size = stat_buf.st_size;
    
    sres = VG_(am_mmap_file_float_valgrind)
-             ( *size, VKI_PROT_READ, fd.val, 0 );
+             ( *size, VKI_PROT_READ, fd.res, 0 );
 
-   VG_(close)(fd.val);
+   VG_(close)(fd.res);
    
    if (sres.isError)
       return 0;
 
-   calccrc = calc_gnu_debuglink_crc32(0, (UChar*)sres.val, *size);
+   calccrc = calc_gnu_debuglink_crc32(0, (UChar*)sres.res, *size);
    if (calccrc != crc) {
-      SysRes res = VG_(am_munmap_valgrind)(sres.val, *size);
+      SysRes res = VG_(am_munmap_valgrind)(sres.res, *size);
       vg_assert(!res.isError);
       if (VG_(clo_verbosity) > 1)
 	 VG_(message)(Vg_DebugMsg, "... CRC mismatch (computed %08x wanted %08x)", calccrc, crc);
       return 0;
    }
    
-   return sres.val;
+   return sres.res;
 }
 
 /*
@@ -825,7 +828,7 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
    oimage = (Addr)NULL;
    if (VG_(clo_verbosity) > 1 || VG_(clo_trace_redir))
       VG_(message)(Vg_DebugMsg, "Reading syms from %s (%p)", 
-                                si->filename, si->start );
+                                si->filename, si->text_start_avma );
 
    /* mmap the object image aboard, so that we can read symbols and
       line number info out of it.  It will be munmapped immediately
@@ -837,17 +840,17 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
       return False;
    }
 
-   n_oimage = VG_(fsize)(fd.val);
+   n_oimage = VG_(fsize)(fd.res);
    if (n_oimage < 0) {
       ML_(symerr)("Can't stat .so/.exe (to determine its size)?!");
-      VG_(close)(fd.val);
+      VG_(close)(fd.res);
       return False;
    }
 
    sres = VG_(am_mmap_file_float_valgrind)
-             ( n_oimage, VKI_PROT_READ, fd.val, 0 );
+             ( n_oimage, VKI_PROT_READ, fd.res, 0 );
 
-   VG_(close)(fd.val);
+   VG_(close)(fd.res);
 
    if (sres.isError) {
       VG_(message)(Vg_UserMsg, "warning: mmap failed on %s", si->filename );
@@ -855,7 +858,12 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
       return False;
    }
 
-   oimage = sres.val;
+   oimage = sres.res;
+
+   if (0) {
+      VG_(printf)("read_elf_debug_info: OIMAGE = %p - %p\n", 
+                  (void*)oimage, (void*)(oimage + (UWord)n_oimage));
+   }
 
    /* Ok, the object image is safely in oimage[0 .. n_oimage-1]. 
       Now verify that it is a valid ELF .so or executable image.
@@ -926,7 +934,7 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
 
 	 if (!offset_set) {
 	    offset_set = True;
-	    offset_oimage = si->start - o_phdr->p_vaddr;
+	    offset_oimage = si->text_start_avma - o_phdr->p_vaddr;
 	    baseaddr = o_phdr->p_vaddr;
 	 }
 
@@ -940,11 +948,11 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
          // Get the data and bss start/size if appropriate
 	 mapped = o_phdr->p_vaddr + offset_oimage;
 	 mapped_end = mapped + o_phdr->p_memsz;
-	 if (si->data_start_vma == 0 &&
+	 if (si->data_start_avma == 0 &&
 	     (o_phdr->p_flags & (PF_R|PF_W|PF_X)) == (PF_R|PF_W)) {
-	    si->data_start_vma = mapped;
-	    si->data_size      = o_phdr->p_filesz;
-	    si->bss_start_vma  = mapped + o_phdr->p_filesz;
+	    si->data_start_avma = mapped;
+	    si->data_size       = o_phdr->p_filesz;
+	    si->bss_start_avma  = mapped + o_phdr->p_filesz;
 	    if (o_phdr->p_memsz > o_phdr->p_filesz)
 	       si->bss_size = o_phdr->p_memsz - o_phdr->p_filesz;
 	    else
@@ -954,23 +962,36 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
          mapped = mapped & ~(VKI_PAGE_SIZE-1);
 	 mapped_end = (mapped_end + VKI_PAGE_SIZE - 1) & ~(VKI_PAGE_SIZE-1);
 
-	 if (VG_(needs).data_syms &&
-	     (mapped >= si->start && mapped <= (si->start+si->size)) &&
-	     (mapped_end > (si->start+si->size))) {
-	    UInt newsz = mapped_end - si->start;
-	    if (newsz > si->size) {
+	 if (VG_(needs).data_syms 
+             && mapped >= si->text_start_avma 
+             && mapped <= (si->text_start_avma + si->text_size)
+             && mapped_end > (si->text_start_avma + si->text_size)) {
+            /* XXX jrs 2007 Jan 11: what's going on here?  If data
+               syms are involved, surely we shouldn't be messing with
+               the segment's text_size unless there is an assumption
+               that the data segment has been mapped immediately after
+               the text segment.  Which doesn't sound good to me. */
+	    UInt newsz = mapped_end - si->text_start_avma;
+	    if (newsz > si->text_size) {
 	       if (0)
 		  VG_(printf)("extending mapping %p..%p %d -> ..%p %d\n", 
-			      si->start, si->start+si->size, si->size,
-			      si->start+newsz, newsz);
+			      si->text_start_avma, 
+                              si->text_start_avma + si->text_size, 
+                              si->text_size,
+			      si->text_start_avma + newsz, newsz);
 
-	       si->size = newsz;
+	       si->text_size = newsz;
 	    }
 	 }
       }
    }
 
-   si->offset = offset_oimage;
+   si->text_bias = offset_oimage;
+
+   if (VG_(clo_verbosity) > 2 || VG_(clo_trace_redir))
+      VG_(message)(Vg_DebugMsg, "   svma %010p, avma %010p", 
+                                si->text_start_avma - si->text_bias,
+                                si->text_start_avma );
 
    /* If, after looking at all the program headers, we still didn't 
       find a soname, add a fake one. */
@@ -993,35 +1014,35 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
    /* Find interesting sections, read the symbol table(s), read any debug
       information */
    {
-      /* Pointers to start of sections (in the oimage, not in the
-	 running image) */
-      UChar*     o_strtab     = NULL; /* .strtab */
-      ElfXX_Sym* o_symtab     = NULL; /* .symtab */
-      UChar*     o_dynstr     = NULL; /* .dynstr */
-      ElfXX_Sym* o_dynsym     = NULL; /* .dynsym */
-      Char*      debuglink    = NULL; /* .gnu_debuglink */
-      UChar*     stab         = NULL; /* .stab         (stabs)  */
-      UChar*     stabstr      = NULL; /* .stabstr      (stabs)  */
-      UChar*     debug_line   = NULL; /* .debug_line   (dwarf2) */
-      UChar*     debug_info   = NULL; /* .debug_info   (dwarf2) */
-      UChar*     debug_abbv   = NULL; /* .debug_abbrev (dwarf2) */
-      UChar*     debug_str    = NULL; /* .debug_str    (dwarf2) */
-      UChar*     dwarf1d      = NULL; /* .debug        (dwarf1) */
-      UChar*     dwarf1l      = NULL; /* .line         (dwarf1) */
-      UChar*     ehframe      = NULL; /* .eh_frame     (dwarf2) */
-      UChar*     opd_filea    = NULL; /* .opd          (dwarf2, ppc64-linux) */
-      UChar*     dummy_filea  = NULL;
+      /* IMAGE addresses: pointers to start of sections (in the
+         oimage, not in the running image) -- image addresses */
+      UChar*     strtab_img      = NULL; /* .strtab */
+      ElfXX_Sym* symtab_img      = NULL; /* .symtab */
+      UChar*     dynstr_img      = NULL; /* .dynstr */
+      ElfXX_Sym* dynsym_img      = NULL; /* .dynsym */
+      Char*      debuglink_img   = NULL; /* .gnu_debuglink */
+      UChar*     stab_img        = NULL; /* .stab         (stabs)  */
+      UChar*     stabstr_img     = NULL; /* .stabstr      (stabs)  */
+      UChar*     debug_line_img  = NULL; /* .debug_line   (dwarf2) */
+      UChar*     debug_info_img  = NULL; /* .debug_info   (dwarf2) */
+      UChar*     debug_abbv_img  = NULL; /* .debug_abbrev (dwarf2) */
+      UChar*     debug_str_img   = NULL; /* .debug_str    (dwarf2) */
+      UChar*     dwarf1d_img     = NULL; /* .debug        (dwarf1) */
+      UChar*     dwarf1l_img     = NULL; /* .line         (dwarf1) */
+      UChar*     ehframe_img     = NULL; /* .eh_frame     (dwarf2) */
+      UChar*     opd_filea_img   = NULL; /* .opd          (dwarf2, ppc64-linux) */
+      UChar*     dummy_filea_img = NULL;
 
-      OffT       o_symtab_offset = offset_oimage;
-      OffT       o_dynsym_offset = offset_oimage;
+      OffT       symtab_offset   = offset_oimage;
+      OffT       dynsym_offset   = offset_oimage;
       OffT       debug_offset    = offset_oimage;
       OffT       opd_offset      = offset_oimage;
 
       /* Section sizes, in bytes */
-      UInt       o_strtab_sz     = 0;
-      UInt       o_symtab_sz     = 0;
-      UInt       o_dynstr_sz     = 0;
-      UInt       o_dynsym_sz     = 0;
+      UInt       strtab_sz       = 0;
+      UInt       symtab_sz       = 0;
+      UInt       dynstr_sz       = 0;
+      UInt       dynsym_sz       = 0;
       UInt       debuglink_sz    = 0;
       UInt       stab_sz         = 0;
       UInt       stabstr_sz      = 0;
@@ -1033,9 +1054,9 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
       UInt       dwarf1l_sz      = 0;
       UInt       ehframe_sz      = 0;
 
-      /* Section virtual addresses */
-      Addr       dummy_vma       = 0;
-      Addr       ehframe_vma     = 0;
+      /* Section actual virtual addresses */
+      Addr       dummy_avma      = 0;
+      Addr       ehframe_avma    = 0;
 
       /* Find all interesting sections */
 
@@ -1073,45 +1094,46 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
 
          /* Nb: must find where .got and .plt sections will be in the
           * executable image, not in the object image transiently loaded. */
-         /*   NAME              SIZE           ADDR_IN_OIMAGE  ADDR_WHEN_MAPPED */
-         FIND(".dynsym",        o_dynsym_sz,   o_dynsym,       dummy_vma)
-         FIND(".dynstr",        o_dynstr_sz,   o_dynstr,       dummy_vma)
-         FIND(".symtab",        o_symtab_sz,   o_symtab,       dummy_vma)
-         FIND(".strtab",        o_strtab_sz,   o_strtab,       dummy_vma)
+         /*   NAME              SIZE           IMAGE addr       AVMA */
+         FIND(".dynsym",        dynsym_sz,     dynsym_img,      dummy_avma)
+         FIND(".dynstr",        dynstr_sz,     dynstr_img,      dummy_avma)
+         FIND(".symtab",        symtab_sz,     symtab_img,      dummy_avma)
+         FIND(".strtab",        strtab_sz,     strtab_img,      dummy_avma)
 
-         FIND(".gnu_debuglink", debuglink_sz,  debuglink,      dummy_vma)
+         FIND(".gnu_debuglink", debuglink_sz,  debuglink_img,   dummy_avma)
 
-         FIND(".stab",          stab_sz,       stab,           dummy_vma)
-         FIND(".stabstr",       stabstr_sz,    stabstr,        dummy_vma)
+         FIND(".stab",          stab_sz,       stab_img,        dummy_avma)
+         FIND(".stabstr",       stabstr_sz,    stabstr_img,     dummy_avma)
 
-         FIND(".debug_line",    debug_line_sz, debug_line,     dummy_vma)
-         FIND(".debug_info",    debug_info_sz, debug_info,     dummy_vma)
-         FIND(".debug_abbrev",  debug_abbv_sz, debug_abbv,     dummy_vma)
-         FIND(".debug_str",     debug_str_sz,  debug_str,      dummy_vma)
+         FIND(".debug_line",    debug_line_sz, debug_line_img,  dummy_avma)
+         FIND(".debug_info",    debug_info_sz, debug_info_img,  dummy_avma)
+         FIND(".debug_abbrev",  debug_abbv_sz, debug_abbv_img,  dummy_avma)
+         FIND(".debug_str",     debug_str_sz,  debug_str_img,   dummy_avma)
 
-         FIND(".debug",         dwarf1d_sz,    dwarf1d,        dummy_vma)
-         FIND(".line",          dwarf1l_sz,    dwarf1l,        dummy_vma)
-         FIND(".eh_frame",      ehframe_sz,    ehframe,        ehframe_vma)
+         FIND(".debug",         dwarf1d_sz,    dwarf1d_img,     dummy_avma)
+         FIND(".line",          dwarf1l_sz,    dwarf1l_img,     dummy_avma)
+         FIND(".eh_frame",      ehframe_sz,    ehframe_img,     ehframe_avma)
 
-         FIND(".got",           si->got_size,  dummy_filea,    si->got_start_vma)
-         FIND(".plt",           si->plt_size,  dummy_filea,    si->plt_start_vma)
-         FIND(".opd",           si->opd_size,  opd_filea,      si->opd_start_vma)
+         FIND(".got",           si->got_size,  dummy_filea_img, si->got_start_avma)
+         FIND(".plt",           si->plt_size,  dummy_filea_img, si->plt_start_avma)
+         FIND(".opd",           si->opd_size,  opd_filea_img,   si->opd_start_avma)
 
 #        undef FIND
       }
          
       /* Did we find a debuglink section? */
-      if (debuglink != NULL) {
-         UInt crc_offset = VG_ROUNDUP(VG_(strlen)(debuglink)+1, 4);
+      if (debuglink_img != NULL) {
+         UInt crc_offset = VG_ROUNDUP(VG_(strlen)(debuglink_img)+1, 4);
          UInt crc;
 
          vg_assert(crc_offset + sizeof(UInt) <= debuglink_sz);
 
          /* Extract the CRC from the debuglink section */
-         crc = *(UInt *)(debuglink + crc_offset);
+         crc = *(UInt *)(debuglink_img + crc_offset);
 
          /* See if we can find a matching debug file */
-         if ((dimage = find_debug_file(si->filename, debuglink, crc, &n_dimage)) != 0) {
+         dimage = find_debug_file(si->filename, debuglink_img, crc, &n_dimage);
+         if (dimage != 0) {
             ehdr = (ElfXX_Ehdr*)dimage;
 
             if (n_dimage >= sizeof(ElfXX_Ehdr) 
@@ -1119,25 +1141,28 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
                 && ehdr->e_phoff + ehdr->e_phnum*sizeof(ElfXX_Phdr) <= n_dimage
                 && ehdr->e_shoff + ehdr->e_shnum*sizeof(ElfXX_Shdr) <= n_dimage)
             {
-               Bool need_symtab = (NULL == o_symtab);
+               Bool need_symtab = (NULL == symtab_img);
+               Bool need_stabs = (NULL == stab_img);
+               Bool need_dwarf2 = (NULL == debug_info_img);
+               Bool need_dwarf1 = (NULL == dwarf1d_img);
 
                for (i = 0; i < ehdr->e_phnum; i++) {
                   ElfXX_Phdr *o_phdr = &((ElfXX_Phdr *)(dimage + ehdr->e_phoff))[i];
                   if (o_phdr->p_type == PT_LOAD) {
-                     offset_dimage = si->start - o_phdr->p_vaddr;
+                     offset_dimage = si->text_start_avma - o_phdr->p_vaddr;
                      break;
                   }
                }
 
                debug_offset = offset_dimage;
                if (need_symtab)
-                  o_symtab_offset = offset_dimage;
+                  symtab_offset = offset_dimage;
 
                shdr = (ElfXX_Shdr*)(dimage + ehdr->e_shoff);
                sh_strtab = (UChar*)(dimage + shdr[ehdr->e_shstrndx].sh_offset);
 
                /* Same deal as previous FIND, except simpler - doesn't
-                  look for vma, only oimage address. */
+                  look for avma, only oimage address. */
 
                /* Find all interesting sections */
                for (i = 0; i < ehdr->e_shnum; i++) {
@@ -1161,16 +1186,17 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
                      } \
                   }
 
-                  FIND(need_symtab, ".symtab",       o_symtab_sz,   o_symtab)
-                  FIND(need_symtab, ".strtab",       o_strtab_sz,   o_strtab)
-                  FIND(1,           ".stab",         stab_sz,       stab)
-                  FIND(1,           ".stabstr",      stabstr_sz,    stabstr)
-                  FIND(1,           ".debug_line",   debug_line_sz, debug_line)
-                  FIND(1,           ".debug_info",   debug_info_sz, debug_info)
-                  FIND(1,           ".debug_abbrev", debug_abbv_sz, debug_abbv)
-                  FIND(1,           ".debug_str",    debug_str_sz,  debug_str)
-                  FIND(1,           ".debug",        dwarf1d_sz,    dwarf1d)
-                  FIND(1,           ".line",         dwarf1l_sz,    dwarf1l)
+                  /*   ??           NAME             SIZE           IMAGE addr */
+                  FIND(need_symtab, ".symtab",       symtab_sz,     symtab_img)
+                  FIND(need_symtab, ".strtab",       strtab_sz,     strtab_img)
+                  FIND(need_stabs,  ".stab",         stab_sz,       stab_img)
+                  FIND(need_stabs,  ".stabstr",      stabstr_sz,    stabstr_img)
+                  FIND(need_dwarf2, ".debug_line",   debug_line_sz, debug_line_img)
+                  FIND(need_dwarf2, ".debug_info",   debug_info_sz, debug_info_img)
+                  FIND(need_dwarf2, ".debug_abbrev", debug_abbv_sz, debug_abbv_img)
+                  FIND(need_dwarf2, ".debug_str",    debug_str_sz,  debug_str_img)
+                  FIND(need_dwarf1, ".debug",        dwarf1d_sz,    dwarf1d_img)
+                  FIND(need_dwarf1, ".line",         dwarf1l_sz,    dwarf1l_img)
 
 #                 undef FIND
                }
@@ -1179,8 +1205,8 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
       }
 
       /* Check some sizes */
-      vg_assert((o_dynsym_sz % sizeof(ElfXX_Sym)) == 0);
-      vg_assert((o_symtab_sz % sizeof(ElfXX_Sym)) == 0);
+      vg_assert((dynsym_sz % sizeof(ElfXX_Sym)) == 0);
+      vg_assert((symtab_sz % sizeof(ElfXX_Sym)) == 0);
 
       /* Read symbols */
       {
@@ -1192,26 +1218,29 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
          read_elf_symtab = read_elf_symtab__normal;
 #        endif
          read_elf_symtab(si, "symbol table",
-                         o_symtab, o_symtab_sz, o_symtab_offset,
-                         o_strtab, o_strtab_sz, opd_filea, opd_offset);
+                         symtab_img, symtab_sz, symtab_offset,
+                         strtab_img, strtab_sz, 
+                         opd_filea_img, opd_offset);
 
          read_elf_symtab(si, "dynamic symbol table",
-                         o_dynsym, o_dynsym_sz, o_dynsym_offset,
-                         o_dynstr, o_dynstr_sz, opd_filea, opd_offset);
+                         dynsym_img, dynsym_sz, dynsym_offset,
+                         dynstr_img, dynstr_sz, 
+                         opd_filea_img, opd_offset);
       }
 
       /* Read .eh_frame (call-frame-info) if any */
-      if (ehframe) {
-         ML_(read_callframe_info_dwarf2) ( si, ehframe, ehframe_sz, ehframe_vma );
+      if (ehframe_img) {
+         ML_(read_callframe_info_dwarf3)
+            ( si, ehframe_img, ehframe_sz, ehframe_avma );
       }
 
       /* Read the stabs and/or dwarf2 debug information, if any.  It
          appears reading stabs stuff on amd64-linux doesn't work, so
          we ignore it. */
 #     if !defined(VGP_amd64_linux)
-      if (stab && stabstr) {
-         ML_(read_debuginfo_stabs) ( si, debug_offset, stab, stab_sz, 
-                                         stabstr, stabstr_sz );
+      if (stab_img && stabstr_img) {
+         ML_(read_debuginfo_stabs) ( si, debug_offset, stab_img, stab_sz, 
+                                         stabstr_img, stabstr_sz );
       }
 #     endif
       /* jrs 2006-01-01: icc-8.1 has been observed to generate
@@ -1219,16 +1248,17 @@ Bool ML_(read_elf_debug_info) ( struct _SegInfo* si )
          debuginfo reading for that reason, but, in
          read_unitinfo_dwarf2, do check that debugstr is non-NULL
          before using it. */
-      if (debug_info && debug_abbv && debug_line /* && debug_str */) {
+      if (debug_info_img && debug_abbv_img && debug_line_img
+                                           /* && debug_str_img */) {
          ML_(read_debuginfo_dwarf2) ( si, debug_offset, 
-                                      debug_info,   debug_info_sz,
-                                      debug_abbv,
-                                      debug_line,   debug_line_sz,
-                                      debug_str );
+                                      debug_info_img,   debug_info_sz,
+                                      debug_abbv_img,
+                                      debug_line_img,   debug_line_sz,
+                                      debug_str_img );
       }
-      if (dwarf1d && dwarf1l) {
-         ML_(read_debuginfo_dwarf1) ( si, dwarf1d, dwarf1d_sz, 
-                                          dwarf1l, dwarf1l_sz );
+      if (dwarf1d_img && dwarf1l_img) {
+         ML_(read_debuginfo_dwarf1) ( si, dwarf1d_img, dwarf1d_sz, 
+                                          dwarf1l_img, dwarf1l_sz );
       }
    }
    res = True;
