@@ -1,7 +1,7 @@
 
 /*--------------------------------------------------------------------*/
 /*--- Format-neutral storage of and querying of info acquired from ---*/
-/*--- ELF/XCOFF stabs/dwarf1/dwarf2 debug info.                    ---*/
+/*--- ELF/XCOFF stabs/dwarf1/dwarf2/dwarf3 debug info.             ---*/
 /*---                                                    storage.c ---*/
 /*--------------------------------------------------------------------*/
 
@@ -9,7 +9,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2006 Julian Seward 
+   Copyright (C) 2000-2007 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -43,6 +43,7 @@
 #include "pub_core_libcbase.h"
 #include "pub_core_libcprint.h"
 #include "pub_core_mallocfree.h"
+#include "pub_core_xarray.h"
 #include "priv_storage.h"          /* self */
 
 
@@ -69,7 +70,7 @@ void ML_(ppSym) ( Int idx, DiSym* sym )
 }
 
 /* Print a call-frame-info summary. */
-void ML_(ppDiCfSI) ( DiCfSI* si )
+void ML_(ppDiCfSI) ( XArray* /* of CfiExpr */ exprs, DiCfSI* si )
 {
 #  define SHOW_HOW(_how, _off)                   \
       do {                                       \
@@ -84,15 +85,34 @@ void ML_(ppDiCfSI) ( DiCfSI* si )
          } else                                  \
          if (_how == CFIR_MEMCFAREL) {           \
             VG_(printf)("*(cfa+%d)", _off);      \
+         } else                                  \
+         if (_how == CFIR_EXPR) {                \
+            VG_(printf)("{");                    \
+            ML_(ppCfiExpr)(exprs, _off);         \
+            VG_(printf)("}");                    \
          } else {                                \
-            VG_(printf)("???");                  \
+            vg_assert(0+0);                      \
          }                                       \
       } while (0)
 
    VG_(printf)("[%p .. %p]: ", si->base, 
                                si->base + (UWord)si->len - 1);
-   VG_(printf)("let cfa=%s+%d", 
-               si->cfa_sprel ? "oldSP" : "oldFP", si->cfa_off);
+   switch (si->cfa_how) {
+      case CFIC_SPREL: 
+         VG_(printf)("let cfa=oldSP+%d", si->cfa_off); 
+         break;
+      case CFIC_FPREL: 
+         VG_(printf)("let cfa=oldFP+%d", si->cfa_off); 
+         break;
+      case CFIC_EXPR: 
+         VG_(printf)("let cfa={"); 
+         ML_(ppCfiExpr)(exprs, si->cfa_off);
+         VG_(printf)("}"); 
+         break;
+      default: 
+         vg_assert(0);
+   }
+
    VG_(printf)(" in RA=");
    SHOW_HOW(si->ra_how, si->ra_off);
    VG_(printf)(" SP=");
@@ -253,13 +273,16 @@ void ML_(addLineInfo) ( struct _SegInfo* si,
        size = 1;
    }
 
-   /* vg_assert(this < si->start + si->size && next-1 >= si->start); */
-   if (this >= si->start + si->size || next-1 < si->start) {
+   /* vg_assert(this < si->text_start_avma + si->size 
+                && next-1 >= si->text_start_avma); */
+   if (this >= si->text_start_avma + si->text_size 
+       || next-1 < si->text_start_avma) {
        if (0)
           VG_(message)(Vg_DebugMsg, 
                        "warning: ignoring line info entry falling "
                        "outside current SegInfo: %p %p %p %p",
-                       si->start, si->start + si->size, 
+                       si->text_start_avma, 
+                       si->text_start_avma + si->text_size, 
                        this, next-1);
        return;
    }
@@ -305,7 +328,7 @@ void ML_(addDiCfSI) ( struct _SegInfo* si, DiCfSI* cfsi )
 
    if (debug) {
       VG_(printf)("adding DiCfSI: ");
-      ML_(ppDiCfSI)(cfsi);
+      ML_(ppDiCfSI)(si->cfsi_exprs, cfsi);
    }
 
    /* sanity */
@@ -319,8 +342,8 @@ void ML_(addDiCfSI) ( struct _SegInfo* si, DiCfSI* cfsi )
    /* Rule out ones which are completely outside the segment.  These
       probably indicate some kind of bug, but for the meantime ignore
       them. */
-   if ( cfsi->base + cfsi->len - 1  <  si->start
-        || si->start + si->size - 1 < cfsi->base ) {
+   if ( cfsi->base + cfsi->len - 1  <  si->text_start_avma
+        || si->text_start_avma + si->text_size - 1  <  cfsi->base ) {
       static Int complaints = 3;
       if (VG_(clo_trace_cfi) || complaints > 0) {
          complaints--;
@@ -330,12 +353,12 @@ void ML_(addDiCfSI) ( struct _SegInfo* si, DiCfSI* cfsi )
                "warning: DiCfSI %p .. %p outside segment %p .. %p",
                cfsi->base, 
                cfsi->base + cfsi->len - 1,
-               si->start,
-               si->start + si->size - 1 
+               si->text_start_avma,
+               si->text_start_avma + si->text_size - 1 
             );
          }
          if (VG_(clo_trace_cfi)) 
-            ML_(ppDiCfSI)(cfsi);
+            ML_(ppDiCfSI)(si->cfsi_exprs, cfsi);
       }
       return;
    }
@@ -359,6 +382,117 @@ void ML_(addDiCfSI) ( struct _SegInfo* si, DiCfSI* cfsi )
 }
 
 
+Int ML_(CfiExpr_Undef)( XArray* dst )
+{
+   CfiExpr e;
+   VG_(memset)( &e, 0, sizeof(e) );
+   e.tag = Cex_Undef;
+   return VG_(addToXA)( dst, &e );
+}
+Int ML_(CfiExpr_Deref)( XArray* dst, Int ixAddr )
+{
+   CfiExpr e;
+   VG_(memset)( &e, 0, sizeof(e) );
+   e.tag = Cex_Deref;
+   e.Cex.Deref.ixAddr = ixAddr;
+   return VG_(addToXA)( dst, &e );
+}
+Int ML_(CfiExpr_Const)( XArray* dst, UWord con )
+{
+   CfiExpr e;
+   VG_(memset)( &e, 0, sizeof(e) );
+   e.tag = Cex_Const;
+   e.Cex.Const.con = con;
+   return VG_(addToXA)( dst, &e );
+}
+Int ML_(CfiExpr_Binop)( XArray* dst, CfiOp op, Int ixL, Int ixR )
+{
+   CfiExpr e;
+   VG_(memset)( &e, 0, sizeof(e) );
+   e.tag = Cex_Binop;
+   e.Cex.Binop.op  = op;
+   e.Cex.Binop.ixL = ixL;
+   e.Cex.Binop.ixR = ixR;
+   return VG_(addToXA)( dst, &e );
+}
+Int ML_(CfiExpr_CfiReg)( XArray* dst, CfiReg reg )
+{
+   CfiExpr e;
+   VG_(memset)( &e, 0, sizeof(e) );
+   e.tag = Cex_CfiReg;
+   e.Cex.CfiReg.reg = reg;
+   return VG_(addToXA)( dst, &e );
+}
+Int ML_(CfiExpr_DwReg)( XArray* dst, Int reg )
+{
+   CfiExpr e;
+   VG_(memset)( &e, 0, sizeof(e) );
+   e.tag = Cex_DwReg;
+   e.Cex.DwReg.reg = reg;
+   return VG_(addToXA)( dst, &e );
+}
+
+static void ppCfiOp ( CfiOp op ) 
+{
+   switch (op) {
+      case Cop_Add: VG_(printf)("+"); break;
+      case Cop_Sub: VG_(printf)("-"); break;
+      case Cop_And: VG_(printf)("&"); break;
+      case Cop_Mul: VG_(printf)("*"); break;
+      default:      vg_assert(0);
+   }
+}
+
+static void ppCfiReg ( CfiReg reg )
+{
+   switch (reg) {
+      case Creg_SP: VG_(printf)("SP"); break;
+      case Creg_FP: VG_(printf)("FP"); break;
+      case Creg_IP: VG_(printf)("IP"); break;
+      default: vg_assert(0);
+   }
+}
+
+void ML_(ppCfiExpr)( XArray* src, Int ix )
+{
+   /* VG_(indexXA) checks for invalid src/ix values, so we can
+      use it indiscriminately. */
+   CfiExpr* e = (CfiExpr*) VG_(indexXA)( src, ix );
+   switch (e->tag) {
+      case Cex_Undef: 
+         VG_(printf)("Undef"); 
+         break;
+      case Cex_Deref: 
+         VG_(printf)("*("); 
+         ML_(ppCfiExpr)(src, e->Cex.Deref.ixAddr);
+         VG_(printf)(")"); 
+         break;
+      case Cex_Const: 
+         VG_(printf)("0x%lx", e->Cex.Const.con); 
+         break;
+      case Cex_Binop: 
+         VG_(printf)("(");
+         ML_(ppCfiExpr)(src, e->Cex.Binop.ixL);
+         VG_(printf)(")");
+         ppCfiOp(e->Cex.Binop.op);
+         VG_(printf)("(");
+         ML_(ppCfiExpr)(src, e->Cex.Binop.ixR);
+         VG_(printf)(")");
+         break;
+      case Cex_CfiReg:
+         ppCfiReg(e->Cex.CfiReg.reg);
+         break;
+      case Cex_DwReg:
+         VG_(printf)("dwr%d", e->Cex.DwReg.reg);
+         break;
+      default: 
+         VG_(core_panic)("ML_(ppCfiExpr)"); 
+         /*NOTREACHED*/
+         break;
+   }
+}
+
+
 /*------------------------------------------------------------*/
 /*--- Canonicalisers                                       ---*/
 /*------------------------------------------------------------*/
@@ -394,7 +528,7 @@ static Int compare_DiSym ( void* va, void* vb )
    so we can misdescribe memcmp() as bcmp()).  This is hard to avoid.
    It's mentioned in the FAQ file.
  */
-static DiSym* prefersym ( DiSym* a, DiSym* b )
+static DiSym* prefersym ( struct _SegInfo* si, DiSym* a, DiSym* b )
 {
    Int lena, lenb;		/* full length */
    Int vlena, vlenb;		/* length without version */
@@ -473,7 +607,7 @@ static void canonicaliseSymtab ( struct _SegInfo* si )
             n_merged++;
             /* merge the two into one */
 	    si->symtab[si->symtab_used++] 
-               = *prefersym(&si->symtab[i], &si->symtab[i+1]);
+               = *prefersym(si, &si->symtab[i], &si->symtab[i+1]);
             i++;
          } else {
             si->symtab[si->symtab_used++] = si->symtab[i];
@@ -496,7 +630,7 @@ static void canonicaliseSymtab ( struct _SegInfo* si )
          continue;
 
       /* There's an overlap.  Truncate one or the other. */
-      if (VG_(clo_trace_symtab)) {
+      if (si->trace_symtab) {
          VG_(printf)("overlapping address ranges in symbol table\n\t");
          ML_(ppSym)( i, &si->symtab[i] );
          VG_(printf)("\t");
@@ -562,7 +696,7 @@ static void canonicaliseSymtab ( struct _SegInfo* si )
    ranges do not overlap.  This facilitates using binary search to map
    addresses to locations when we come to query the table.
 */
-static Int compare_RiLoc ( void* va, void* vb ) 
+static Int compare_DiLoc ( void* va, void* vb ) 
 {
    DiLoc* a = (DiLoc*)va;
    DiLoc* b = (DiLoc*)vb;
@@ -583,7 +717,7 @@ static void canonicaliseLoctab ( struct _SegInfo* si )
 
    /* Sort by start address. */
    VG_(ssort)(si->loctab, si->loctab_used, 
-                          sizeof(*si->loctab), compare_RiLoc);
+                          sizeof(*si->loctab), compare_DiLoc);
 
    /* If two adjacent entries overlap, truncate the first. */
    for (i = 0; i < ((Int)si->loctab_used)-1; i++) {
@@ -680,7 +814,7 @@ static void canonicaliseCFI ( struct _SegInfo* si )
          si->cfsi_maxaddr = here_max;
    }
 
-   if (VG_(clo_trace_cfi))
+   if (si->trace_cfi)
       VG_(printf)("canonicaliseCfiSI: %d entries, %p .. %p\n", 
                   si->cfsi_used,
 	          si->cfsi_minaddr, si->cfsi_maxaddr);
