@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2006 Julian Seward
+   Copyright (C) 2000-2007 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -83,7 +83,7 @@ typedef
       Bool libc_freeres;
       Bool core_errors;
       Bool tool_errors;
-      Bool basic_block_discards;
+      Bool superblock_discards;
       Bool command_line_options;
       Bool client_requests;
       Bool syscall_wrapper;
@@ -91,6 +91,7 @@ typedef
       Bool data_syms;
       Bool malloc_replacement;
       Bool xml_output;
+      Bool final_IR_tidy_pass;
    } 
    VgNeeds;
 
@@ -105,8 +106,8 @@ typedef struct {
    // Basic functions
    void  (*tool_pre_clo_init) (void);
    void  (*tool_post_clo_init)(void);
-   IRBB* (*tool_instrument)   (VgCallbackClosure*,
-                               IRBB*, 
+   IRSB* (*tool_instrument)   (VgCallbackClosure*,
+                               IRSB*, 
                                VexGuestLayout*, VexGuestExtents*, 
                                IRType, IRType);
    void  (*tool_fini)         (Int);
@@ -117,6 +118,7 @@ typedef struct {
    // VG_(needs).tool_errors
    Bool  (*tool_eq_Error)                    (VgRes, Error*, Error*);
    void  (*tool_pp_Error)                    (Error*);
+   Bool  tool_show_ThreadIDs_for_errors;
    UInt  (*tool_update_extra)                (Error*);
    Bool  (*tool_recognised_suppression)      (Char*, Supp*);
    Bool  (*tool_read_extra_suppression_info) (Int, Char*, Int, Supp*);
@@ -124,8 +126,8 @@ typedef struct {
    Char* (*tool_get_error_name)              (Error*);
    void  (*tool_print_extra_suppression_info)(Error*);
 
-   // VG_(needs).basic_block_discards
-   void (*tool_discard_basic_block_info)(Addr64, VexGuestExtents);
+   // VG_(needs).superblock_discards
+   void (*tool_discard_superblock_info)(Addr64, VexGuestExtents);
 
    // VG_(needs).command_line_options
    Bool (*tool_process_cmd_line_option)(Char*);
@@ -155,6 +157,9 @@ typedef struct {
    void* (*tool_realloc)             (ThreadId, void*, SizeT);
    SizeT tool_client_redzone_szB;
 
+   // VG_(needs).final_IR_tidy_pass
+   IRSB* (*tool_final_IR_tidy_pass)  (IRSB*);
+
    // -- Event tracking functions ------------------------------------
    void (*track_new_mem_startup)     (Addr, SizeT, Bool, Bool, Bool);
    void (*track_new_mem_stack_signal)(Addr, SizeT);
@@ -200,14 +205,12 @@ typedef struct {
    void (*track_post_reg_write)(CorePart, ThreadId,        OffT, SizeT);
    void (*track_post_reg_write_clientcall_return)(ThreadId, OffT, SizeT, Addr);
 
-   void (*track_thread_run)(ThreadId);
+   void (*track_start_client_code)(ThreadId, ULong);
+   void (*track_stop_client_code) (ThreadId, ULong);
 
-   void (*track_post_thread_create)(ThreadId, ThreadId);
-   void (*track_post_thread_join)  (ThreadId, ThreadId);
-
-   void (*track_pre_mutex_lock)   (ThreadId, void*);
-   void (*track_post_mutex_lock)  (ThreadId, void*);
-   void (*track_post_mutex_unlock)(ThreadId, void*);
+   void (*track_pre_thread_ll_create)(ThreadId, ThreadId);
+   void (*track_pre_thread_first_insn)(ThreadId);
+   void (*track_pre_thread_ll_exit)  (ThreadId);
 
    void (*track_pre_deliver_signal) (ThreadId, Int sigNo, Bool);
    void (*track_post_deliver_signal)(ThreadId, Int sigNo);
