@@ -8,7 +8,7 @@
    This file is part of MemCheck, a heavyweight Valgrind tool for
    detecting memory errors.
 
-   Copyright (C) 2000-2006 Julian Seward 
+   Copyright (C) 2000-2007 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -56,7 +56,7 @@ typedef
    struct _MC_Chunk {
       struct _MC_Chunk* next;
       Addr         data;            // ptr to actual block
-      SizeT        size : (sizeof(UWord)*8)-2; // size requested; 30 or 62 bits
+      SizeT        szB : (sizeof(UWord)*8)-2; // size requested; 30 or 62 bits
       MC_AllocKind allockind : 2;   // which wrapper did the allocation
       ExeContext*  where;           // where it was allocated
    }
@@ -86,6 +86,10 @@ extern void MC_(destroy_mempool) ( Addr pool );
 extern void MC_(mempool_alloc)   ( ThreadId tid, Addr pool,
                                    Addr addr, SizeT size );
 extern void MC_(mempool_free)    ( Addr pool, Addr addr );
+extern void MC_(mempool_trim)    ( Addr pool, Addr addr, SizeT size );
+extern void MC_(move_mempool)    ( Addr poolA, Addr poolB );
+extern void MC_(mempool_change)  ( Addr pool, Addr addrA, Addr addrB, SizeT size );
+extern Bool MC_(mempool_exists)  ( Addr pool );
 
 extern MC_Chunk* MC_(get_freed_list_head)( void );
 
@@ -115,28 +119,6 @@ extern void  MC_(__builtin_vec_delete) ( ThreadId tid, void* p );
 extern void* MC_(realloc)              ( ThreadId tid, void* p, SizeT new_size );
 
 
-/*------------------------------------------------------------*/
-/*--- Errors and suppressions                              ---*/
-/*------------------------------------------------------------*/
-
-/* Extra info for overlap errors */
-typedef
-   struct {
-      Addr src;
-      Addr dst;
-      Int  len;   // -1 if unused
-   }
-   OverlapExtra;
-
-extern void MC_(record_free_error)            ( ThreadId tid, Addr a ); 
-extern void MC_(record_illegal_mempool_error) ( ThreadId tid, Addr a );
-extern void MC_(record_freemismatch_error)    ( ThreadId tid, Addr a,
-                                                MC_Chunk* mc );
-extern Bool MC_(record_leak_error)            ( ThreadId tid, 
-                                                void* leak_extra,
-                                                ExeContext* where,
-                                                Bool print_record );
-
 /*------------------------------------------------------------*/
 /*--- Profiling of memory events                           ---*/
 /*------------------------------------------------------------*/
@@ -197,6 +179,21 @@ extern HChar* MC_(event_ctr_name)[N_PROF_EVENTS];
 /*--- Leak checking                                        ---*/
 /*------------------------------------------------------------*/
 
+/* A block is either 
+   -- Proper-ly reached; a pointer to its start has been found
+   -- Interior-ly reached; only an interior pointer to it has been found
+   -- Unreached; so far, no pointers to any part of it have been found. 
+   -- IndirectLeak; leaked, but referred to by another leaked block
+*/
+typedef 
+   enum { 
+      Unreached    =0, 
+      IndirectLeak =1,
+      Interior     =2, 
+      Proper       =3
+  }
+  Reachedness;
+
 /* For VALGRIND_COUNT_LEAKS client request */
 extern SizeT MC_(bytes_leaked);
 extern SizeT MC_(bytes_indirect);
@@ -204,9 +201,6 @@ extern SizeT MC_(bytes_dubious);
 extern SizeT MC_(bytes_reachable);
 extern SizeT MC_(bytes_suppressed);
 
-/* For leak checking */
-extern void MC_(pp_LeakError)(void* extra);
-                           
 typedef
    enum {
       LC_Off,
@@ -215,12 +209,44 @@ typedef
    }
    LeakCheckMode;
 
+/* A block record, used for generating err msgs. */
+typedef
+   struct _LossRecord {
+      struct _LossRecord* next;
+      /* Where these lost blocks were allocated. */
+      ExeContext*  allocated_at;
+      /* Their reachability. */
+      Reachedness  loss_mode;
+      /* Number of blocks and total # bytes involved. */
+      SizeT        total_bytes;
+      SizeT        indirect_bytes;
+      UInt         num_blocks;
+   }
+   LossRecord;
+
 extern void MC_(do_detect_memory_leaks) (
           ThreadId tid, LeakCheckMode mode,
           Bool (*is_within_valid_secondary) ( Addr ),
           Bool (*is_valid_aligned_word)     ( Addr )
        );
 
+extern void MC_(pp_LeakError)(UInt n_this_record, UInt n_total_records,
+                              LossRecord* l);
+                          
+
+/*------------------------------------------------------------*/
+/*--- Errors and suppressions                              ---*/
+/*------------------------------------------------------------*/
+
+extern void MC_(record_free_error)            ( ThreadId tid, Addr a ); 
+extern void MC_(record_illegal_mempool_error) ( ThreadId tid, Addr a );
+extern void MC_(record_freemismatch_error)    ( ThreadId tid, MC_Chunk* mc );
+extern Bool MC_(record_leak_error)            ( ThreadId tid,
+                                                UInt n_this_record,
+                                                UInt n_total_records,
+                                                LossRecord* lossRecord,
+                                                Bool print_record );
+
 /*------------------------------------------------------------*/
 /*--- Command line options + defaults                      ---*/
 /*------------------------------------------------------------*/
@@ -229,7 +255,7 @@ extern void MC_(do_detect_memory_leaks) (
 extern Bool MC_(clo_partial_loads_ok);
 
 /* Max volume of the freed blocks queue. */
-extern Int MC_(clo_freelist_vol);
+extern Long MC_(clo_freelist_vol);
 
 /* Do leak check at exit?  default: NO */
 extern LeakCheckMode MC_(clo_leak_check);
@@ -256,6 +282,15 @@ extern Bool MC_(clo_workaround_gcc296_bugs);
  * default: YES */
 extern Bool MC_(clo_undef_value_errors);
 
+/* Fill malloc-d/free-d client blocks with a specific value?  -1 if
+   not, else 0x00 .. 0xFF indicating the fill value to use.  Can be
+   useful for causing programs with bad heap corruption to fail in
+   more repeatable ways.  Note that malloc-filled and free-filled
+   areas are still undefined and noaccess respectively.  This merely
+   causes them to contain the specified values. */
+extern Int MC_(clo_malloc_fill);
+extern Int MC_(clo_free_fill);
+
 
 /*------------------------------------------------------------*/
 /*--- Instrumentation                                      ---*/
@@ -288,12 +323,15 @@ extern void MC_(helperc_MAKE_STACK_UNINIT) ( Addr base, UWord len );
 
 /* Functions defined in mc_translate.c */
 extern
-IRBB* MC_(instrument) ( VgCallbackClosure* closure,
-                        IRBB* bb_in, 
+IRSB* MC_(instrument) ( VgCallbackClosure* closure,
+                        IRSB* bb_in, 
                         VexGuestLayout* layout, 
                         VexGuestExtents* vge,
                         IRType gWordTy, IRType hWordTy );
 
+extern
+IRSB* MC_(final_tidy) ( IRSB* );
+
 #endif /* ndef __MC_INCLUDE_H */
 
 /*--------------------------------------------------------------------*/
